/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef XSID_WB_DEBUG
#include <iomanip>
#endif
#include <fstream>
#include <iostream>
using namespace std;

#include "IniFile.h"
#include "Config.h"

IniFile::IniFile(const int newMaxLineLen) : TextFile(newMaxLineLen)
{
    ///qDeleteAll (groups.begin(), groups.end());
    groups.clear();
    
    curGroup = 0;
    curEntry = 0;
}

IniFile::~IniFile()
{
}

void IniFile::setHeader(const char* comment)
{
    configHeaderComment = comment;
}

void IniFile::setGroup(const char* name)
{ ff();
    if ( curGroup!=0 && curGroup->name==name )
        return;
#ifdef XSID_WB_DEBUG
    cout << "::setGroup(" << name << ")" << endl;
#endif
    if ( !haveGroup(name) )
    {
        // Group not found. Create it.
        Group* ns = new Group(name);
        groups.append(*ns);
        ns=&groups.last();             // without this did not works
        curGroup = ns;
#ifdef XSID_WB_DEBUG
        cout << "Group " << name << " not found. Created it [." << &(*ns) << "]" << endl;
	ff();
#endif
    }
#ifdef XSID_WB_DEBUG
    cout << curGroup->name.ascii() << endl;
#endif
}

// Overloaded.
void IniFile::setEntry(const char* key, const char* value)
{
    if ( haveEntry(key) )
    {
        // Key found. Assign/copy new value.
        curEntry->value = value;
    }
    else
    {
#ifdef XSID_WB_DEBUG
        cout << "Entry " << key << " not found. Created it." << endl;
#endif
        // Key not found. Create key/value pair.
        curEntry = new Entry(key,value);
        curGroup->entries.append(*curEntry);
    }
#ifdef XSID_WB_DEBUG
    cout << "key = " << curEntry->key.ascii() << " ## " << "value = " << curEntry->value.ascii() << endl;
#endif
}

// Overloaded.
void IniFile::setEntry(const char* key, int value)
{
    QString tmp;
    tmp.setNum(value);
    setEntry(key,tmp.toLatin1().data());
}

// Overloaded.
void IniFile::setEntry(const char* key, long int value)
{
    QString tmp;
    tmp.setNum(value);
    setEntry(key,tmp.toLatin1().data());
}

// Overloaded.
void IniFile::setEntry(const char* key, float value)
{
    QString tmp;
    tmp.setNum(value);
    setEntry(key,tmp.toLatin1().data());
}

// Overloaded.
bool IniFile::getValue(const char* key, QString& value)
{
#ifdef XSID_WB_DEBUG
    cout << "get string value: key = " << key << endl;
#endif  
    if ( haveEntry(key) )
    {
        value = curEntry->value;
#ifdef XSID_WB_DEBUG
    cout << "value = " << value.ascii() << endl;
#endif 
        return true;
    }
    else
    {
        return false;
    }
}

// Overloaded.
bool IniFile::getValue(const char* key, float& value)
{
#ifdef XSID_WB_DEBUG
    cout << "get float value: key = " << key << endl;
#endif   
    if ( haveEntry(key) )
    {
        value = curEntry->value.toFloat();
#ifdef XSID_WB_DEBUG
    cout << "value = " << value << endl;
#endif 
        return true;
    }
    else
    {
        return false;
    }
}

// Overloaded.
bool IniFile::getValue(const char* key, int& value)
{
#ifdef XSID_WB_DEBUG
    cout << "get int value: key = " << key << endl;
#endif    
    long int i;
    bool ret = getValue(key,i);  // get long int value
    if ( ret )
        value = i;               // truncate it
        
 #ifdef XSID_WB_DEBUG
    cout << "value = " << value << endl;
#endif        
    return ret;
    
}

// Overloaded.
bool IniFile::getValue(const char* key, long int& value)
{
#ifdef XSID_WB_DEBUG
    cout << "get long int value: key = " << key << endl;
#endif   
    if ( haveEntry(key) )
    {
        value = curEntry->value.toLong();
#ifdef XSID_WB_DEBUG
    cout << "value = " << value << endl;
#endif 
        return true;
    }
    else
    {
        return false;
    }
}

// Overloaded.
bool IniFile::getValue(const char* key, bool& value)
{
 #ifdef XSID_WB_DEBUG
    cout << "get bool value: key = " << key << endl;
#endif  
    long int i;
    bool ret = getValue(key,i);  // get long int value
    if ( ret )
        value = (i!=0);          // convert to int

#ifdef XSID_WB_DEBUG
    cout << "value = " << value << endl;
#endif 
    return ret;
}

const QString& IniFile::getString(const char* key)
{
    if ( haveEntry(key) )
        return curEntry->value;
    else
        return empty;
}

bool IniFile::getBool(const char* key)
{
    if ( haveEntry(key) )
        return (curEntry->value.toInt()!=0);
    else
        return false;
}

long int IniFile::getLongInt(const char* key)
{
    if ( haveEntry(key) )
        return curEntry->value.toLong();
    else
        return 0;
}

bool IniFile::haveGroup(const char* name)
{
    QListIterator<Group> it(groups);
    while( it.hasNext() ) 
    {
        Group* value=(Group *)&it.next();
        if ( value->name==name )
        {
            curGroup = value;  
            return true;  // group found
        }
    }
   
    return false;  // group not found
}

bool IniFile::haveEntry(const char* key)
{
    if ( curGroup==0 )
        return false;
    QListIterator<Entry> it(curGroup->entries);
    while( it.hasNext() )
    {
        Entry* value=(Entry *)&it.next();
        if ( value->key==key )
        {
            curEntry = value;
            return true;  // entry found
        }
    }
    return false;
}

void IniFile::removeEntry(const char* key)
{
    if ( haveEntry(key) )
    {
        curGroup->entries.removeAll(*curEntry);
    }
    else
    {
#ifdef XSID_WB_DEBUG
        cout << "Not found. Nothing to remove." << endl;
#endif
    }
}

bool IniFile::load(const char* fileName)
{
    open(fileName);
    while (status && isGood && !isEOF())  // line-by-line loop
    {
        getLine();
        // Skip blank and comment lines.
        while (status && !isEOF() && isBlank() || isComment())
        {
            getLine();
        };
        if ( isEOF() )
            break;
        // Evaluate line.
#ifdef XSID_WB_DEBUG
        cout << "Line " << getLineNum() << ", " << getLineLen() << ": ";
        cout << getLineBuf() << endl;
        cout << "ParseBuf: " << getParseBuf() << endl;
#endif
        if ( lineBuf[0]=='[' )  // group start?
        {
            QString tmp = lineBuf+1;
            int closePos = tmp.indexOf(']');
            if ( closePos>0 )
            {
                tmp.truncate(closePos);
                setGroup(tmp.toLatin1().data());
            }
        }
        else  // some other line content
        {
            QString tmp = lineBuf;
            int equalPos = tmp.indexOf('=');
            if ( equalPos>0 )
            {
                setEntry(tmp.left(equalPos).toLatin1().data(),
                         tmp.mid(equalPos+1).toLatin1().data());
            }
        }
    }
    close();
    return isGood;
}

bool IniFile::save(const char* fileName)
{
    bool wasSuccess = false;

#ifdef XSID_HAVE_IOS_BIN
    ofstream toFile(fileName,ios::out|ios::bin|ios::trunc);
#else
    ofstream toFile(fileName,ios::out|ios::binary|ios::trunc);
#endif
    if ( !toFile.fail() )
    {
        if ( !configHeaderComment.isEmpty() )
            toFile << configHeaderComment.toStdString () << endl;

        QListIterator<Group> secit(groups);
        while( secit.hasNext() )
        {
	    Group* group=(Group *)&secit.next();
            toFile << '[' << group->name.toStdString() << ']' << endl;

            QListIterator<Entry> setit(group->entries);
            while ( setit.hasNext() )
            {
                Entry* entry=(Entry *)&setit.next();       
                toFile << entry->key.toStdString() << '=' 
                    << entry->value.toStdString() << endl;
            }
            toFile << endl;
        }
        toFile.close();
        wasSuccess = !toFile.fail();
    }
    return wasSuccess;
}
