/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "EmuWrapper.h"

#ifdef SID_WITH_SIDPLAY1
// An instance of this structure is used to transport emulator settings
// to and from the interface class.
struct emuConfig
{
	uword_emuwt frequency;       // [frequency] = Hz, min=4000, max=48000
	int bitsPerSample;     // see below, ``Sample precision''
	int sampleFormat;      // see below, ``Sample encoding''
	int channels;          // see below, ``Number of physical audio channels''
	int sidChips;          // --- unsupported ---
	int volumeControl;     // see below, ``Volume control modes''

	bool mos8580;          // true, false (just the waveforms)
	bool measuredVolume;   // true, false

	bool emulateFilter;    // true, false
	float filterFs;        // 1.0 <= Fs
	float filterFm;        // Fm != 0
	float filterFt;        //
	
	int memoryMode;        // MPU_BANK_SWITCHING, MPU_TRANSPARENT_ROM,
                           // MPU_PLAYSID_ENVIRONMENT
	
	int clockSpeed;        // SIDTUNE_CLOCK_PAL, SIDTUNE_CLOCK_NTSC

	bool forceSongSpeed;   // true, false
	
	//
	// Working, but experimental.
	//
	int digiPlayerScans;   // 0=off, number of C64 music player calls used to
	                       // scan it for PlaySID Extended SID Register usage.

	int autoPanning;       // see below, ``Auto-panning''
};
#endif

QString EmuWrapper::versionString;
int EmuWrapper::engines = 0;
QMutex EmuWrapper::emuMutex;

EmuConfig EmuWrapper::myEmuConfig;
EmuConfig EmuWrapper::myEmuConfigDefault;

EmuWrapper::EmuWrapper()
{
    if ( engines==0 )
    {

        ++engines;
    }
}

EmuWrapper::~EmuWrapper()
{
    if ( --engines==0 )
    {
    ///SidDrivers::getDriver()->~SidLibrary();
    }
}

void EmuWrapper::initSong(int song)
{
    emuMutex.lock();
    SidDrivers::getDriver()->initSong(song);
    emuMutex.unlock();
}

void EmuWrapper::setConfig(const EmuConfig& inConfig)
{
    emuMutex.lock();
    SidDrivers::getDriver()->setConfig(inConfig);
    
    /// we must register the same config for all available drivers
    ///SidDrivers::List::iterator it;
    ///for ( it = SidDrivers::begin(); it != SidDrivers::end(); it++ ) {
    ///   it->pDriver->setConfig(inConfig);
    /// }
    emuMutex.unlock();
}

const EmuConfig& EmuWrapper::getConfig()
{
    emuMutex.lock();
    myEmuConfig=SidDrivers::getDriver()->getConfig();
    emuMutex.unlock();
    return myEmuConfig;
}

const EmuConfig& EmuWrapper::getConfigDefault()
{
    return myEmuConfigDefault;
}

void EmuWrapper::resetSecondsThisSong()
{
    emuMutex.lock();
    SidDrivers::getDriver()->resetSecondsThisSong();
    emuMutex.unlock();
}

int EmuWrapper::getSecondsThisSong()
{
    emuMutex.lock();
    int secs = SidDrivers::getDriver()->getSecondsThisSong();
    emuMutex.unlock();
    return secs;
}

int EmuWrapper::getSecondsTotal()
{
    emuMutex.lock();
    int secs = SidDrivers::getDriver()->getSecondsTotal();
    emuMutex.unlock();
    return secs;
}

const char* EmuWrapper::getVersionString()
{
    emuMutex.lock();
    versionString = SidDrivers::getDriver()->getVersionString();
    emuMutex.unlock();    
    return versionString.toLatin1();
}

const char* EmuWrapper::getSpeedString()
{
    emuMutex.lock();    
    const char* res=SidDrivers::getDriver()->getSpeedString();    
    emuMutex.unlock();
    return res;
}

const char* EmuWrapper::getSidModelString()
{
    emuMutex.lock();
    const char* res= SidDrivers::getDriver()->getSidModelString();
    emuMutex.unlock();
    return res;
}

void EmuWrapper::stop()
{
    emuMutex.lock();
    SidDrivers::getDriver()->stop();
    emuMutex.unlock();
}

void EmuWrapper::fillBuffer(SidLibrary* pSidTune, ubyte_emuwt* pBuffer, udword_emuwt bufferSize)
{
    emuMutex.lock();
    SidDrivers::getDriver()->fillBuffer(pBuffer, bufferSize);
    emuMutex.unlock();
}

void EmuWrapper::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total)
{
    emuMutex.lock();
    SidDrivers::getDriver()->setVoiceVolume(voice,left,right,total);
    emuMutex.unlock();
}

int EmuWrapper::getCia1TA()
{
    emuMutex.lock(); 
    int res=SidDrivers::getDriver()->getCia1TA();
    emuMutex.unlock();
    
    return res;
}

int EmuWrapper::getClockSpeed()
{
    emuMutex.lock();
    int res=SidDrivers::getDriver()->getClockSpeed();
    emuMutex.unlock();
    
    return res;
}

const char* EmuWrapper::getClockFreqString()
{
    emuMutex.lock();
    const char* res=SidDrivers::getDriver()->getClockFreqString();
    emuMutex.unlock();

    return res;
}

