/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LIBSIDPLAY1
#define LIBSIDPLAY1

#include "LibSidplay1.h"
#include <sidplay/sidtune.h>
#include <sidplay/myendian.h>
#include "../emucfg.h"

const char sidtune_text_songNumberExceed[] = "WARNING: Selected song number was too high";
const char sidtune_text_PAL_VBI[] = "50 Hz VBI (PAL)";
const char sidtune_text_PAL_CIA[] = "CIA 1 Timer A (PAL)";
const char sidtune_text_NTSC_VBI[] = "60 Hz VBI (NTSC)";
const char sidtune_text_NTSC_CIA[] = "CIA 1 Timer A (NTSC)";
    
/**
 * Manage LibSidplay 1 library
 */ 
class LibSidplay1_impl : public SidLibrary, public sidTune {
  protected:
    emuEngine* pEmuEngine;
    sidTuneInfo* pSidInfo;
     QString* pDigest;
     
  private:         
    uword selectSongPublic(uword selectedSong);   
       
  public:
    
    /**
     * Construct the library 
     */
    LibSidplay1_impl();  
    
    /**
     * Deconstruct the library
     */ 
    ~LibSidplay1_impl();    
    
    /**
     * Get configuration
     * 
     * @return the configuration
     */ 
    const EmuConfig& getConfig();
    
    /**
     * Get default configuration
     * 
     * @return the default configuration
     */
    const EmuConfig& getConfigDefault(); 
    
    /**
     * Set the configuration to use
     * 
     * @param inConfig input configuration
     */ 
    void setConfig(const EmuConfig& inConfig);      
    
    /**
     * Init with the given song
     * 
     * @param song the given number of song to init
     */ 
    void initSong(int song);    
    
    /**
     * Fill the buffer with played music
     * 
     * @param pBuffer the buffer to fill
     * @param bufferSize the size of the buffer
     */ 
    void fillBuffer(void* pBuffer, unsigned long int bufferSize);    
    
    /**
     * Stop the engine
     */
    void stop();
    
    /**
     * Set the volume for the given voice
     * 
     * @param voice the number of voice
     * @param left left volume
     * @param right right volume
     * @param total total volume
     */
    void setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total);   
    
    /**
     * Reset the seconds of this song
     */
    void resetSecondsThisSong();
    
    /**
     * Get the seconds of this song
     * 
     * @return the seconds of this song
     */    
    int getSecondsThisSong();  
    
    /**
     * Get the total seconds
     * 
     * @return the total seconds
     */    
    int getSecondsTotal();    
    
    /**
     * Get string version
     * 
     * @return the string version
     */
    const char* getVersionString();  
    
    /**
     * Get speed string
     * 
     * @return the speed string
     */
    const char* getSpeedString(); 
    
    /**
     * Get sid model string
     * 
     * @return the sid model string
     */    
    const char* getSidModelString();    
    
    /**
     * Get clock frequnecy string
     * 
     * @return the clock frequency string
     */      
    const char* getClockFreqString();    
    
    /**
     * Get last CIA1 TA value of C64
     * 
     * @return the last CIA1 TA value
     */    
    int getCia1TA();  
    
    /**
     * Get clock speed     
     *  
     * @return the clock speed (-1=not implemented)
     */
    int getClockSpeed();    
    
    /**
     * Get the name of Sid Library managed
     * 
     * @return the name of sid library managed
     */
    const char* getName() const {
      return LIB_L1;
    }    
    
    /**
     * Get the enum of Sid Library managed
     * 
     * @return the enum of sid library managed
     */
    const int getEnum() const {
      return EmuConfig::L1;
    }    
        
        
    /**
     * Get the max songs
     * 
     * @return the max songs
     */ 
    int getMaxSongs() const;   
    
    /**
     * Get the max sid file length
     * 
     * @return the max sid file length
     */ 
    unsigned long int getMaxSidFileLen() const;   
    
    /**
     * Open the give file
     * 
     * @param fileName the name of file
     * @return true if operation is ok 
     */     
    bool open(const QString& fileName);
    
    /**
     * Load the file in the given buffer
     * 
     * @paraM buf the buffer
     * @param bufLen the length of buffer
     * @return true if the operatuion is ok
     */ 
    bool load(void*, unsigned long int);   
    
    /**
     * Save the file as PSID
     * 
     * @param fileName the fileName to used
     * @param overWrite true if over write
     */ 
    bool savePSID(const QString&, bool);    
    
    /**
     * Update information
     */ 
    void updateInfo();  
    
    /**
    * Get the load address
    * 
    * @return the load address
    */ 
   int getLoadAddr() const;

   /**
    * Get the init address
    * 
    * @return the init address
    */
   int getInitAddr() const;

   /**
    * Get the play address
    * 
    * @return the play address
    */ 
   int getPlayAddr() const;

   /**
    * Get the number of songs
    * 
    * @return the number of songs
    */ 
   int getSongs() const;

   /**
    * Get the current song
    * 
    * @return the current song
    */ 
   int getCurrentSong() const;

   /**
    * Get thwe number of starting song
    *  
    * @return the starting song
    */ 
   int getStartSong() const;

   /**
    * Get the info string of the given index
    * 
    * @param i the index
    * @return the info string
    */ 
   const char* getInfoString(int i) const;

   /**
    * Get numebr of info string
    * 
    * @return the number of imput string
    */ 
   int getInfoStringsNum() const;

   /**
    * Get the status string
    * 
    * @return the status string
    */ 
   const char* getStatusString() const;

   /**
    * Get the speed string
    * 
    * @return the speed string
    */
   const char* getSpeedString2() const;

   /**
    * Get the format string
    * 
    * @return the formal string
    */ 
   const char* getFormatString() const; 
   
   /**
    * Get status
    * 
    * @return the status
    */ 
   bool getStatus();  
   
   /**
    * Get MD5
    * 
    * @return the MD5
    */    
   const QString& getMD5_Digest();   
   
    /**
     * Create MD5
     * 
     * @param myMD5 MD5 to create
     */
    void createMD5(MD5& myMD5);
    
   /**
    * Get the data buffer
    * 
    * @return the data buffer
    */ 
   const unsigned char *getDataBuffer();

   /**
    * Get the data length
    * 
    * @return the data length
    */
   int getDataLength(); 
};    


/**
 * Construct the library
 */ 
LibSidplay1_impl::LibSidplay1_impl() : sidTune(0) {
  if ( pEmuEngine==0 ) {
    pEmuEngine = new emuEngine;
    pEmuEngine->getConfig(*myEmuConfigDefault.pConfig1);
    myEmuConfig = myEmuConfigDefault;
  }
  
  pSidInfo = new sidTuneInfo;
  pDigest = new QString;
}

/**
 * Deconstruct the library
 */ 
LibSidplay1_impl::~LibSidplay1_impl() {
  delete pEmuEngine;
  delete pSidInfo;
  delete pDigest;
}

/**
 * Get configuration
 * 
 * @return the configuration
 */ 
const EmuConfig& LibSidplay1_impl::getConfig() {
  pEmuEngine->getConfig(*myEmuConfig.pConfig1);

  return myEmuConfig; 
}
    
/**
 * Get default configuration
 * 
 * @return the default configuration
 */
const EmuConfig& LibSidplay1_impl::getConfigDefault() {
  return myEmuConfigDefault;  
}

/**
 * Set the configuration to use
 * 
 * @param inConfig input configuration
 */ 
void LibSidplay1_impl::setConfig(const EmuConfig& inConfig) {
  myEmuConfig = inConfig;
  pEmuEngine->setConfig(*myEmuConfig.pConfig1);
  pEmuEngine->getConfig(*myEmuConfig.pConfig1);
}


/**
 * Init with the given song
 * 
 * @param song the given number of song to init
 */ 
void LibSidplay1_impl::initSong(int song) {
  sidEmuInitializeSong(*pEmuEngine,*this,song);
}

/**
 * Fill the buffer with played music
 * 
 * @param pBuffer the buffer to fill
 * @param bufferSize the size of the buffer
 */ 
void LibSidplay1_impl::fillBuffer(void* pBuffer, unsigned long buffer bufferSize) {
  sidEmuFillBuffer(*pEmuEngine, *this, (ubyte_emuwt)pBuffer, (udword_emuwt)bufferSize);
}

/**
 * Stop the engine
 */
void LibSidplay1_impl::stop() {
} 

/**
 * Set the volume for the given voice
 * 
 * @param voice the number of voice
 * @param left left volume
 * @param right right volume
 * @param total total volume
 */
void LibSidplay1_impl::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
  pEmuEngine->setVoiceVolume(voice,left,right,total);
}

/**
 * Reset the seconds of this song
 */
void LibSidplay1_impl::resetSecondsThisSong() {  
  pEmuEngine->resetSecondsThisSong();
}

/**
 * Get the seconds of this song
 * 
 * @return the seconds of this song
 */    
int LibSidplay1_impl::getSecondsThisSong() {
  return pEmuEngine->getSecondsThisSong();
}

/**
 * Get the total seconds
 * 
 * @return the total seconds
 */    
int LibSidplay1_impl::getSecondsTotal() {
  return pEmuEngine->getSecondsTotal();
} 

/**
 * Get string version
 * 
 * @return the string version
 */
const char* LibSidplay1_impl::getVersionString() {
  versionString = "SID player library   Version ";
  versionString += pEmuEngine->getVersionString();
  versionString += "\nhttp://www.geocities.com/SiliconValley/Lakes/5147/sidplay/";
  
  return versionString.toLatin1();
}

/**
 * Get speed string
 * 
 * @return the speed string
 */
const char* LibSidplay1_impl::getSpeedString() {
  return "";
}

/**
 * Get sid model string
 * 
 * @return the sid model string
 */    
const char* LibSidplay1_impl::getSidModelString() {
  return "";
}

/**
 * Get clock frequnecy string
 * 
 * @return the clock frequency string
 */      
const char* LibSidplay1_impl::getClockFreqString() {
  return "";  
}

/**
 * Get last CIA1 TA value of C64
 * 
 * @return the last CIA1 TA value
 */    
int LibSidplay1_impl::getCia1TA() {
  return -1;
}

/**
 * Get clock speed     
 *  
 * @return the clock speed (-1=not implemented)
 */
int LibSidplay1_impl::getClockSpeed()
{
  return -1;
}



/**
 * Get the max songs
 * 
 * @return the max songs
 */ 
int LibSidplay1_impl::getMaxSongs() const {
  return classMaxSongs;
}

/**
 * Get the max sid file length
 * 
 * @return the max sid file length
 */ 
unsigned long int LibSidplay1_impl::getMaxSidFileLen() const {
  return maxSidtuneFileLen;
}

    
/**
 * Open the give file
 * 
 * @param fileName the name of file
 * @return true if operation is ok 
 */ 
bool LibSidplay1_impl::open(const QString& fileName) {
  bool ret = sidTune::open(fileName.toLatin1());
  updateInfo();
  return ret;  
}

/**
 * Load the file in the given buffer
 * 
 * @paraM buf the buffer
 * @param bufLen the length of buffer
 * @return true if the operatuion is ok
 */ 
bool LibSidplay1_impl::load(void* buf, unsigned long int bufLen) {
  bool ret = sidTune::load((const ubyte_emuwt*)buf,(udword_emuwt)bufLen);
  updateInfo();
  return ret;  
}

/**
 * Save the file as PSID
 * 
 * @param fileName the fileName to used
 * @param overWrite true if over write
 */ 
bool LibSidplay1_impl::savePSID(const QString& fileName, bool overWrite) {
  return sidTune::savePSIDfile(fileName.toLatin1(),overWrite);
}

/**
 * Update information
 */ 
void  LibSidplay1_impl::updateInfo() {
  sidTune::getInfo(*pSidInfo); 
}  

/**
 * Get the load address
 * 
 * @return the load address
 */ 
int LibSidplay1_impl::getLoadAddr() const {
  return pSidInfo->loadAddr;
}

/**
 * Get the init address
 * 
 * @return the init address
 */
int LibSidplay1_impl::getInitAddr() const {
    return pSidInfo->initAddr;
}

/**
 * Get the play address
 * 
 * @return the play address
 */ 
int LibSidplay1_impl::getPlayAddr() const {
    return pSidInfo->playAddr;
}

/**
 * Get the number of songs
 * 
 * @return the number of songs
 */ 
int LibSidplay1_impl::getSongs() const {
    return pSidInfo->songs;
}

/**
 * Get the current song
 * 
 * @return the current song
 */ 
int LibSidplay1_impl::getCurrentSong() const {
    return pSidInfo->currentSong;
}

/**
 * Get thwe number of starting song
 * 
 * @return the starting song
 */ 
int LibSidplay1_impl::getStartSong() const {
    return pSidInfo->startSong;
}

/**
 * Get the info string of the given index
 * 
 * @param i the index
 * @return the info string
 */ 
const char* LibSidplay1_impl::getInfoString(int i) const {
    return pSidInfo->infoString[i];
}

/**
 * Get numebr of info string
 * 
 * @return the number of imput string
 */ 
int LibSidplay1_impl::getInfoStringsNum() const {
    return pSidInfo->numberOfInfoStrings;
}

/**
 * Get the status string
 * 
 * @return the status string
 */ 
const char* LibSidplay1_impl::getStatusString() const {
    return pSidInfo->statusString;
}

/**
 * Get the speed string
 * 
 * @return the speed string
 */
const char* LibSidplay1_impl::getSpeedString2() const {
    return pSidInfo->speedString;
}

/**
 * Get the format string
 * 
 * @return the formal string
 */
const char* LibSidplay1_impl::getFormatString() const {    
  return pSidInfo->formatString;
}

/**
 * Get status
 * 
 * @return the status
 */ 
bool LibSidplay1_impl::getStatus() {
  return sidTune::getStatus();
}

/**
 * Get MD5
 * 
 * @return the MD5
 */    
const QString& LibSidplay1_impl::getMD5_Digest() {
  MD5 myMD5;
  createMD5(myMD5);
  myMD5.finish();
  // Print fingerprint.
  QString tmp;
  pDigest->truncate(0);
  for (int di = 0; di < 16; ++di)
    *pDigest += tmp.sprintf("%02x",(int)myMD5.getDigest()[di]);
  return *pDigest;  
}

/**
 * Create MD5
 * 
 * @param myMD5 MD5 to create
 */
void LibSidplay1_impl::createMD5(MD5& myMD5) {
  if (isCached && status) {
    // Include C64 data.
    myMD5.append(cachePtr+fileOffset,info.c64dataLen);
    ubyte tmp[2];
    
    // Include INIT and PLAY address.
    writeLEword(tmp,info.initAddr);
    myMD5.append(tmp,sizeof(tmp));
    writeLEword(tmp,info.playAddr);
    myMD5.append(tmp,sizeof(tmp));
    // Include number of songs.
    writeLEword(tmp,info.songs);
    myMD5.append(tmp,sizeof(tmp));
    
    // Include song speed for each song.
    for (uword s = 1; s <= info.songs; s++) {
      selectSongPublic(s);
      myMD5.append(&info.songSpeed,sizeof(info.songSpeed));
    }
    
    // Deal with PSID v2NG clock speed flags: Let only NTSC
    // clock speed change the MD5 fingerprint. That way the
    // fingerprint of a PAL-speed sidtune in PSID v1, v2, and
    // PSID v2NG format is the same.
    if ( info.clock == SIDTUNE_CLOCK_NTSC )
      myMD5.append(&info.clock,sizeof(info.clock));

    // NB! If the fingerprint is used as an index into a
    // song-lengths database or cache, modify above code to
    // allow for PSID v2NG files which have clock speed set to
    // SIDTUNE_CLOCK_ANY. If the SID player program fully
    // supports the SIDTUNE_CLOCK_ANY setting, a sidtune could
    // either create two different fingerprints depending on
    // the clock speed chosen by the player, or there could be
    // two different values stored in the database/cache.
  }  
} 

uword LibSidplay1_impl::selectSongPublic(uword selectedSong) {
  // Determine and set starting song number.
  if (selectedSong == 0) {
    selectedSong = info.startSong;
  } else if ((selectedSong > info.songs) || (selectedSong > classMaxSongs)) {
      info.statusString = sidtune_text_songNumberExceed;
      selectedSong = info.startSong;
    }
  info.lengthInSeconds = songLength[selectedSong-1];
  // Retrieve song speed definition.
  info.songSpeed = songSpeed[selectedSong-1];
  info.clockSpeed = clockSpeed[selectedSong-1];
  // Assign song speed description string depending on clock speed.
  // Final description will be available after song init only.
  if (info.clockSpeed == SIDTUNE_CLOCK_PAL ||
      info.clockSpeed == SIDTUNE_CLOCK_UNKNOWN ) {
    if (info.songSpeed == SIDTUNE_SPEED_VBI) {
      info.speedString = sidtune_text_PAL_VBI;
    } else {
        info.speedString = sidtune_text_PAL_CIA;
      }
    //if (info.clockSpeed == SIDTUNE_CLOCK_NTSC)  
  } else {
      if (info.songSpeed == SIDTUNE_SPEED_VBI) {
        info.speedString = sidtune_text_NTSC_VBI;
      } else {
          info.speedString = sidtune_text_NTSC_CIA;
        }
    }
  return (info.currentSong=selectedSong);
}

/**
 * Get the data buffer
 * 
 * @return the data buffer
 */ 
const unsigned char *LibSidplay1_impl::getDataBuffer() {
  return NULL;
}

/**
 * Get the data length
 * 
 * @return the data length
 */
int LibSidplay1_impl::getDataLength() {
  return 0;
}




LibSidplay1::LibSidplay1(): pimpl(new LibSidplay1_impl) {
}

LibSidplay1::~LibSidplay1() {
}

const EmuConfig& LibSidplay1::getConfig() { 
  return pimpl->getConfig(); 
  
}

const EmuConfig& LibSidplay1::getConfigDefault() {
  return pimpl->getConfigDefault();   
}

void LibSidplay1::setConfig(const EmuConfig& inConfig) {
  pimpl->setConfig(inConfig);  
} 

void LibSidplay1::initSong(int song) {
  pimpl->initSong(song);
} 

void LibSidplay1::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
  pimpl->fillBuffer(pBuffer, bufferSize);
}

void LibSidplay1::stop() {
  pimpl->stop();
}

void LibSidplay1::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
  pimpl->setVoiceVolume(voice, left, right, total);
}

void LibSidplay1::resetSecondsThisSong() {
  pimpl->resetSecondsThisSong();
}

int LibSidplay1::getSecondsThisSong() {
  return pimpl->getSecondsThisSong();
}

int LibSidplay1::getSecondsTotal() {
  return pimpl->getSecondsTotal();
}

const char* LibSidplay1::getVersionString() {
  return pimpl->getVersionString();
}

const char* LibSidplay1::getSpeedString() {
  return pimpl->getSpeedString();
}

const char* LibSidplay1::getSidModelString() {
  return pimpl->getSidModelString();
}

const char* LibSidplay1::getClockFreqString() {
  return pimpl->getClockFreqString();
}

int LibSidplay1::getCia1TA() {
  return pimpl->getCia1TA();
}

int LibSidplay1::getClockSpeed() {
  return pimpl->getClockSpeed();
}

void LibSidplay1::createMD5(MD5& myMD5) {
  pimpl->createMD5(myMD5);
}

const unsigned char *LibSidplay1::getDataBuffer() {
  return pimpl->getDataBuffer(); 
}

int LibSidplay1::getDataLength() {
  return pimpl->getDataLength();
}

int LibSidplay1::getMaxSongs() const {
  return pimpl->getMaxSongs();
}

unsigned long int LibSidplay1::getMaxSidFileLen() const {
  return pimpl->getMaxSidFileLen();
}

bool LibSidplay1::open(const QString& fileName) {
  return pimpl->open(fileName);
}

bool LibSidplay1::load(void* buf, unsigned long int bufLen) {
  return pimpl->load(buf, bufLen);
}

bool LibSidplay1::savePSID(const QString& fileName, bool overWrite) {
 return pimpl->savePSID(fileName, overWrite);
}

void LibSidplay1::updateInfo() {
 return pimpl->updateInfo();
}

int LibSidplay1::getLoadAddr() const {
  return pimpl->getLoadAddr();
}

int LibSidplay1::getInitAddr() const {
  return pimpl->getInitAddr();
}

int LibSidplay1::getPlayAddr() const {
  return pimpl->getPlayAddr();
}

int LibSidplay1::getSongs() const {
  return pimpl->getSongs();
}

int LibSidplay1::getCurrentSong() const {
  return pimpl->getCurrentSong();
}

int LibSidplay1::getStartSong() const {
  return pimpl->getStartSong();
}

const char* LibSidplay1::getInfoString(int i) const {
  return pimpl->getInfoString(i);
}

int LibSidplay1::getInfoStringsNum() const {
  return pimpl->getInfoStringsNum();
}

const char* LibSidplay1::getStatusString() const {
  return pimpl->getStatusString();
}

const char* LibSidplay1::getSpeedString2() const {
 return pimpl->getSpeedString2(); 
}

const char* LibSidplay1::getFormatString() const {
 return pimpl->getFormatString();
}

bool LibSidplay1::getStatus() {
  return pimpl->getStatus();
}

const QString& LibSidplay1::getMD5_Digest() {
  return pimpl->getMD5_Digest();
}   





#endif  /* LIBSIDPLAY1 */
