/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LIBSIDPLAY1_H
#define LIBSIDPLAY1_H

#include "SidLibrary.h"
#include "EmuConfig.h"
#include "../MD5.h"
#include <tr1/memory>
#include <sidplay/sidtune.h>

class LibSidplay1_impl;

/**
 * Manage LibSidplay 1 library
 */ 
class LibSidplay1 : public SidLibrary {
  private:         
    std::tr1::shared_ptr<LibSidplay1_impl> pimpl;
       
  public:
    
    /**
     * Construct the library 
     */
    LibSidplay1();  
    
    /**
     * Deconstruct the library
     */ 
    ~LibSidplay1();    
    
    /**
     * Get configuration
     * 
     * @return the configuration
     */ 
    const EmuConfig& getConfig();
    
    /**
     * Get default configuration
     * 
     * @return the default configuration
     */
    const EmuConfig& getConfigDefault(); 
    
    /**
     * Set the configuration to use
     * 
     * @param inConfig input configuration
     */ 
    void setConfig(const EmuConfig& inConfig);      
    
    /**
     * Init with the given song
     * 
     * @param song the given number of song to init
     */ 
    void initSong(int song);    
    
    /**
     * Fill the buffer with played music
     * 
     * @param pBuffer the buffer to fill
     * @param bufferSize the size of the buffer
     */ 
    void fillBuffer(void* pBuffer, unsigned long int bufferSize);    
    
    /**
     * Stop the engine
     */
    void stop();
    
    /**
     * Set the volume for the given voice
     * 
     * @param voice the number of voice
     * @param left left volume
     * @param right right volume
     * @param total total volume
     */
    void setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total);   
    
    /**
     * Reset the seconds of this song
     */
    void resetSecondsThisSong();
    
    /**
     * Get the seconds of this song
     * 
     * @return the seconds of this song
     */    
    int getSecondsThisSong();  
    
    /**
     * Get the total seconds
     * 
     * @return the total seconds
     */    
    int getSecondsTotal();    
    
    /**
     * Get string version
     * 
     * @return the string version
     */
    const char* getVersionString();  
    
    /**
     * Get speed string
     * 
     * @return the speed string
     */
    const char* getSpeedString(); 
    
    /**
     * Get sid model string
     * 
     * @return the sid model string
     */    
    const char* getSidModelString();    
    
    /**
     * Get clock frequnecy string
     * 
     * @return the clock frequency string
     */      
    const char* getClockFreqString();    
    
    /**
     * Get last CIA1 TA value of C64
     * 
     * @return the last CIA1 TA value
     */    
    int getCia1TA();  
    
    /**
     * Get clock speed     
     *  
     * @return the clock speed (-1=not implemented)
     */
    int getClockSpeed();    
    
    /**
     * Get the name of Sid Library managed
     * 
     * @return the name of sid library managed
     */
    const char* getName() const {
      return LIB_L1;
    }    
    
    /**
     * Get the enum of Sid Library managed
     * 
     * @return the enum of sid library managed
     */
    const int getEnum() const {
      return EmuConfig::L1;
    }    
        
        
    /**
     * Get the max songs
     * 
     * @return the max songs
     */ 
    int getMaxSongs() const;   
    
    /**
     * Get the max sid file length
     * 
     * @return the max sid file length
     */ 
    unsigned long int getMaxSidFileLen() const;   
    
    /**
     * Open the give file
     * 
     * @param fileName the name of file
     * @return true if operation is ok 
     */     
    bool open(const QString& fileName);
    
    /**
     * Load the file in the given buffer
     * 
     * @paraM buf the buffer
     * @param bufLen the length of buffer
     * @return true if the operatuion is ok
     */ 
    bool load(void*, unsigned long int);   
    
    /**
     * Save the file as PSID
     * 
     * @param fileName the fileName to used
     * @param overWrite true if over write
     */ 
    bool savePSID(const QString&, bool);    
    
    /**
     * Update information
     */ 
    void updateInfo();  
    
    /**
    * Get the load address
    * 
    * @return the load address
    */ 
   int getLoadAddr() const;

   /**
    * Get the init address
    * 
    * @return the init address
    */
   int getInitAddr() const;

   /**
    * Get the play address
    * 
    * @return the play address
    */ 
   int getPlayAddr() const;

   /**
    * Get the number of songs
    * 
    * @return the number of songs
    */ 
   int getSongs() const;

   /**
    * Get the current song
    * 
    * @return the current song
    */ 
   int getCurrentSong() const;

   /**
    * Get thwe number of starting song
    *  
    * @return the starting song
    */ 
   int getStartSong() const;

   /**
    * Get the info string of the given index
    * 
    * @param i the index
    * @return the info string
    */ 
   const char* getInfoString(int i) const;

   /**
    * Get numebr of info string
    * 
    * @return the number of imput string
    */ 
   int getInfoStringsNum() const;

   /**
    * Get the status string
    * 
    * @return the status string
    */ 
   const char* getStatusString() const;

   /**
    * Get the speed string
    * 
    * @return the speed string
    */
   const char* getSpeedString2() const;

   /**
    * Get the format string
    * 
    * @return the formal string
    */ 
   const char* getFormatString() const; 
   
   /**
    * Get status
    * 
    * @return the status
    */ 
   bool getStatus();  
   
   /**
    * Get MD5
    * 
    * @return the MD5
    */    
   const QString& getMD5_Digest();   
   
    /**
     * Create MD5
     * 
     * @param myMD5 MD5 to create
     */
    void createMD5(MD5& myMD5);
    
   /**
    * Get the data buffer
    * 
    * @return the data buffer
    */ 
   const unsigned char *getDataBuffer();

   /**
    * Get the data length
    * 
    * @return the data length
    */
   int getDataLength(); 
};

#endif  /* LIBSIDPLAY1_H */