/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LIBSIDPLAY2
#define LIBSIDPLAY2


#include <sidplay/sidplay2.h>
#include <sidplay/builders/resid.h>
#include <sidplay/SidTune.h>
#include "../emucfg2.h"
#include "LibSidplay2.h"


/**
 * Manage LibSidplay 2 library
 */ 
class LibSidplay2_impl : public SidLibrary, public  SidTune {
  protected:
    sidplay2* pEmuEngine;
    ReSIDBuilder* pEmuSid;
    sid2_config_t sid2_cfg;
    SidTuneInfo* pSidInfo;
    QString* pDigest;
    
  private:
    inline void writeLEword(unsigned char ptr[2], int someWord) {
      ptr[0] = (someWord & 0xFF);
      ptr[1] = (someWord >> 8);
    }
    
  public:
    
    /**
     * Construct the library 
     */
    LibSidplay2_impl();
    
    /**
     * Deconstruct the library
     */ 
    ~LibSidplay2_impl();
    
    /**
     * Get configuration
     * 
     * @return the configuration
     */ 
    const EmuConfig& getConfig();
    
    /**
     * Get default configuration
     * 
     * @return the default configuration
     */
    const EmuConfig& getConfigDefault();   
    
    /**
     * Set the configuration to use
     * 
     * @param inConfig input configuration
     */ 
    void setConfig(const EmuConfig& inConfig);     
    
    /**
     * Init with the given song
     * 
     * @param song the given number of song to init
     */ 
    void initSong(int song);  
    
    /**
     * Fill the buffer with played music
     * 
     * @param pBuffer the buffer to fill
     * @param bufferSize the size of the buffer
     */ 
    void fillBuffer(void* pBuffer, unsigned long bufferSize);    
    
    /**
     * Stop the engine
     */
    void stop();   
    
    /**
     * Set the volume for the given voice
     * 
     * @param voice the number of voice
     * @param left left volume
     * @param right right volume
     * @param total total volume
     */
    void setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total);
    
    /**
     * Reset the seconds of this song
     */
    void resetSecondsThisSong();  
         
    /**
     * Get the seconds of this song
     * 
     * @return the seconds of this song
     */    
    int getSecondsThisSong();   
    
    /**
     * Get the total seconds
     * 
     * @return the total seconds
     */    
    int getSecondsTotal();    
    
    /**
     * Get string version
     * 
     * @return the string version
     */
    const char* getVersionString();     
    
    /**
     * Get speed string
     * 
     * @return the speed string
     */
    const char* getSpeedString();    
    
    /**
     * Get sid model string
     * 
     * @return the sid model string
     */    
    const char* getSidModelString();  
        
    /**
     * Get clock frequnecy string
     * 
     * @return the clock frequency string
     */      
    const char* getClockFreqString();      
    
    /**
     * Get last CIA1 TA value of C64
     * 
     * @return the last CIA1 TA value
     */    
    int getCia1TA();  
    
    /**
     * Get clock speed     
     *  
     * @return the clock speed (-1=not implemented)
     */
    int getClockSpeed();    
    
    /**
     * Get the name of Sid Library managed
     * 
     * @return the name of sid library managed
     */
    const char* getName() const {
      return LIB_L2;
    }    
    
    /**
     * Get the enum of Sid Library managed
     * 
     * @return the enum of sid library managed
     */
    const int getEnum() const {
      return EmuConfig::L2;
    }    
    
    
    /**
     * Get the max songs
     * 
     * @return the max songs
     */ 
    int getMaxSongs() const; 
    
    /**
     * Get the max sid file length
     * 
     * @return the max sid file length
     */ 
    unsigned long int getMaxSidFileLen() const;  
        
    /**
     * Open the give file
     * 
     * @param fileName the name of file
     * @return true if operation is ok 
     */ 
    bool open(const QString& fileName);    
            
    /**
     * Load the file in the given buffer
     * 
     * @param buf the buffer
     * @param bufLen the length of buffer
     * @return true if the operation is ok
     */ 
    bool load(void*, unsigned long int);  
    
    /**
     * Save the file as PSID
     * 
     * @param fileName the fileName to used
     * @param overWrite true if over write
     */ 
    bool savePSID(const QString&, bool);   
    
    /**
     * Update information
     */ 
    void updateInfo();   
    
   /**
    * Get the load address
    * 
    * @return the load address
    */ 
   int getLoadAddr() const;

   /**
    * Get the init address
    * 
    * @return the init address
    */
   int getInitAddr() const;

   /**
    * Get the play address
    * 
    * @return the play address
    */ 
   int getPlayAddr() const;

   /**
    * Get the number of songs
    * 
    * @return the number of songs
    */ 
   int getSongs() const;

   /**
    * Get the current song
    * 
    * @return the current song
    */ 
   int getCurrentSong() const;

   /**
    * Get thwe number of starting song
    *  
    * @return the starting song
    */ 
   int getStartSong() const;

   /**
    * Get the info string of the given index
    * 
    * @param i the index
    * @return the info string
    */ 
   const char* getInfoString(int i) const;

   /**
    * Get numebr of info string
    * 
    * @return the number of imput string
    */ 
   int getInfoStringsNum() const;

   /**
    * Get the status string
    * 
    * @return the status string
    */ 
   const char* getStatusString() const;

   /**
    * Get the speed string
    * 
    * @return the speed string
    */
   const char* getSpeedString2() const;

   /**
    * Get the format string
    * 
    * @return the formal string
    */ 
   const char* getFormatString() const;
   
   /**
    * Get status
    * 
    * @return the status
    */ 
   bool getStatus();

   /**
    * Get MD5
    * 
    * @return the MD5
    */    
   const QString& getMD5_Digest();   
   
    /**
     * Create MD5
     * 
     * @param myMD5 MD5 to create
     */
    void createMD5(MD5& myMD5);
    
   /**
    * Get the data buffer
    * 
    * @return the data buffer
    */ 
   const unsigned char *getDataBuffer();

   /**
    * Get the data length
    * 
    * @return the data length
    */
   int getDataLength();    
};

/**
 * Construct the library
 */ 
LibSidplay2_impl::LibSidplay2_impl():SidTune(0)  {
  pEmuEngine = new sidplay2;
  pEmuSid = new ReSIDBuilder("XSIDPLAY reSID");
  sid2_cfg = pEmuEngine->config();
  sid2_cfg.sidSamples = false;
  myEmuConfig.pConfig2->emulateFilter = true;
  if (*pEmuSid) {  // TODO: nothrow above
    // Builder initialisation ok.
    pEmuSid->create( (pEmuEngine->info()).maxsids );
    sid2_cfg.sidEmulation = pEmuSid;
    pEmuEngine->config(sid2_cfg);
  }
  myEmuConfigDefault = getConfig ();
  
  pSidInfo = new SidTuneInfo;
  pDigest = new QString;
}

/**
 * Deconstruct the library
 */ 
LibSidplay2_impl::~LibSidplay2_impl() {
  delete pEmuSid;
  delete pEmuEngine;
  delete pSidInfo;
  delete pDigest;
}

/**
 * Get configuration
 * 
 * @return the configuration
 */ 
const EmuConfig& LibSidplay2_impl::getConfig() {
  // Convert config to sidplay1.
  sid2_cfg = pEmuEngine->config();
  myEmuConfig.pConfig2->frequency  = (int) sid2_cfg.frequency;
  myEmuConfig.pConfig2->bitsPerSample = sid2_cfg.precision;
  myEmuConfig.pConfig2->sampleFormat = SIDEMU_SIGNED_PCM_L2;
  if (sid2_cfg.precision == 8)
    myEmuConfig.pConfig2->sampleFormat = SIDEMU_UNSIGNED_PCM_L2;
  myEmuConfig.pConfig2->channels = SIDEMU_MONO_L2;
  if (sid2_cfg.playback == sid2_stereo)
    myEmuConfig.pConfig2->channels = SIDEMU_STEREO_L2;
  myEmuConfig.pConfig2->mos8580 = ( sid2_cfg.sidDefault == SID2_MOS8580 );
  myEmuConfig.pConfig2->forceSidModel = ( sid2_cfg.sidModel != SID2_MODEL_CORRECT );
  myEmuConfig.pConfig2->clockSpeed = SIDTUNE_CLOCK_PAL;
  if (sid2_cfg.clockDefault == SID2_CLOCK_NTSC)
    myEmuConfig.pConfig2->clockSpeed = SIDTUNE_CLOCK_NTSC;
  myEmuConfig.pConfig2->forceSongSpeed = sid2_cfg.clockForced;

  switch (sid2_cfg.environment) {
    case sid2_envPS:
      myEmuConfig.pConfig2->memoryMode = MPU_PLAYSID_ENVIRONMENT_L2;
      break;
    case sid2_envTP:
      myEmuConfig.pConfig2->memoryMode = MPU_TRANSPARENT_ROM_L2;
      break;
    case sid2_envBS:
    case sid2_envR:
    default:
      myEmuConfig.pConfig2->memoryMode = MPU_BANK_SWITCHING_L2;
      break;
  }

  return myEmuConfig; 
}
    
/**
 * Get default configuration
 * 
 * @return the default configuration
 */
const EmuConfig& LibSidplay2_impl::getConfigDefault() {
  return myEmuConfigDefault;  
}

/**
 * Set the configuration to use
 * 
 * @param inConfig input configuration
 */ 
void LibSidplay2_impl::setConfig(const EmuConfig& inConfig) {
    // make a copy of it
    myEmuConfig=inConfig;
    
    // Convert config for sidplay2.
    sid2_cfg.frequency = (uint_least32_t) inConfig.pConfig2->frequency;
    sid2_cfg.precision = inConfig.pConfig2->bitsPerSample;
    sid2_cfg.playback = sid2_mono;
    if (inConfig.pConfig2->channels == SIDEMU_STEREO_L2)
        sid2_cfg.playback = sid2_stereo;
    sid2_cfg.sidModel = SID2_MODEL_CORRECT;
    if ( inConfig.pConfig2->mos8580 )
        sid2_cfg.sidDefault = SID2_MOS8580;
    else
        sid2_cfg.sidDefault = SID2_MOS6581;
    if ( inConfig.pConfig2->forceSidModel )
        sid2_cfg.sidModel = sid2_cfg.sidDefault;
    sid2_cfg.sidSamples = true;
    sid2_cfg.clockSpeed = SID2_CLOCK_CORRECT;
    if (inConfig.pConfig2->clockSpeed == SIDTUNE_CLOCK_NTSC)
        sid2_cfg.clockDefault = SID2_CLOCK_NTSC;
    else
        sid2_cfg.clockDefault = SID2_CLOCK_PAL;
    if ( sid2_cfg.clockForced = inConfig.pConfig2->forceSongSpeed )
        sid2_cfg.clockSpeed = sid2_cfg.clockDefault;

    switch (inConfig.pConfig2->memoryMode)
    {
    case MPU_PLAYSID_ENVIRONMENT_L2:
        sid2_cfg.environment = sid2_envPS;
        break;
    case MPU_TRANSPARENT_ROM_L2:
        sid2_cfg.environment = sid2_envTP;
        break;
    case MPU_BANK_SWITCHING_L2:
    default:
        sid2_cfg.environment = sid2_envBS;
        break;
    }

    pEmuSid->sampling (sid2_cfg.frequency);
    pEmuSid->filter   (inConfig.pConfig2->emulateFilter);

    pEmuEngine->config(sid2_cfg);
    sid2_cfg = pEmuEngine->config();  
}

/**
 * Init with the given song
 * 
 * @param song the given number of song to init
 */ 
void LibSidplay2_impl::initSong(int song) {
  SidTune::selectSong(song);
  pEmuEngine->load(this);
}

/**
 * Fill the buffer with played music
 * 
 * @param pBuffer the buffer to fill
 * @param bufferSize the size of the buffer
 */ 
void LibSidplay2_impl::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
  pEmuEngine->play((ubyte_emuwt*)pBuffer,(udword_emuwt)bufferSize);
}

/**
 * Set the volume for the given voice
 * 
 * @param voice the number of voice
 * @param left left volume
 * @param right right volume
 * @param total total volume
 */
void LibSidplay2_impl::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
}  

/**
 * Reset the seconds of this song
 */
void LibSidplay2_impl::resetSecondsThisSong() {  
}

/**
 * Get the seconds of this song
 * 
 * @return the seconds of this song
 */    
int LibSidplay2_impl::getSecondsThisSong() {
  return pEmuEngine->time() / pEmuEngine->timebase();
}
 
/**
 * Get the total seconds
 * 
 * @return the total seconds
 */    
int LibSidplay2_impl::getSecondsTotal() {
  return pEmuEngine->mileage() / pEmuEngine->timebase();
}      

/**
 * Get string version
 * 
 * @return the string version
 */
const char* LibSidplay2_impl::getVersionString() {
  sid2_info_t info;
  info = pEmuEngine->info();
  
  versionString = "SID player library   Version ";
  versionString += info.version;
  versionString += "\nby Simon White <sidplay2@email.com>\n";
  versionString += "http://sidplay2.sourceforge.net\n\n";
  versionString += pEmuSid->credits();
  
  return versionString.toLatin1();
}

/**
 * Get speed string
 * 
 * @return the speed string
 */
const char* LibSidplay2_impl::getSpeedString() {
  sid2_info_t info = pEmuEngine->info();
  return info.tuneInfo->speedString;
}

/**
 * Get sid model string
 * 
 * @return the sid model string
 */    
const char* LibSidplay2_impl::getSidModelString() {
  sid2_info_t info = pEmuEngine->info();
  if ( info.tuneInfo->sidModel == SID2_MOS8580 ) return "MOS 8580";
  else return "MOS 6581";
}

/**
 * Get clock frequnecy string
 * 
 * @return the clock frequency string
 */      
const char* LibSidplay2_impl::getClockFreqString() {
  sid2_info_t info = pEmuEngine->info();

  switch (info.tuneInfo->clockSpeed) {
    case SIDTUNE_CLOCK_UNKNOWN:
      return "Unknown";
    case SIDTUNE_CLOCK_PAL:
      return "PAL";
    case SIDTUNE_CLOCK_NTSC:
      return "NTSC";
    case SIDTUNE_CLOCK_ANY:
      return "ANY";
  }
  return "";  
} 
 
/**
 * Stop the engine
 */
void LibSidplay2_impl::stop() {
  pEmuEngine->stop();
}  

/**
 * Get last CIA1 TA value of C64
 * 
 * @return the last CIA1 TA value
 */    
int LibSidplay2_impl::getCia1TA() {
  return pEmuEngine->cia1_last_ta();
}

/**
 * Get clock speed     
 *  
 * @return the clock speed (-1=not implemented)
 */
int LibSidplay2_impl::getClockSpeed()
{
  sid2_info_t info = pEmuEngine->info();
  return info.tuneInfo->clockSpeed;
}



/**
 * Get the max songs
 * 
 * @return the max songs
 */ 
int LibSidplay2_impl::getMaxSongs() const {
  return SIDTUNE_MAX_SONGS;
}

/**
 * Get the max sid file length
 * 
 * @return the max sid file length
 */ 
unsigned long int LibSidplay2_impl::getMaxSidFileLen() const {
  return SIDTUNE_MAX_FILELEN;
}

/**
 * Open the give file
 * 
 * @param fileName the name of file
 * @return true if operation is ok 
 */ 
bool LibSidplay2_impl::open(const QString& fileName) {
  bool ret = SidTune::load(fileName.toLatin1());
  updateInfo();
  return ret;  
}

/**
 * Load the file in the given buffer
 * 
 * @param buf the buffer
 * @param bufLen the length of buffer
 * @return true if the operation is ok
 */ 
bool LibSidplay2_impl::load(void* buf, unsigned long int bufLen) {
  bool ret =  SidTune::read((const ubyte_emuwt*)buf,(udword_emuwt)bufLen);
  updateInfo();
  return ret;  
}

/**
 * Save the file as PSID
 * 
 * @param fileName the fileName to used
 * @param overWrite true if over write
 */ 
bool LibSidplay2_impl::savePSID(const QString& fileName, bool overWrite) {
  return SidTune::savePSIDfile(fileName.toLatin1(),overWrite);
}

/**
 * Update information
 */ 
void LibSidplay2_impl::updateInfo() {
  SidTune::getInfo(*pSidInfo); 
}

/**
 * Get the load address
 * 
 * @return the load address
 */ 
int LibSidplay2_impl::getLoadAddr() const {
  return pSidInfo->loadAddr;
}

/**
 * Get the init address
 * 
 * @return the init address
 */
int LibSidplay2_impl::getInitAddr() const {
    return pSidInfo->initAddr;
}

/**
 * Get the play address
 * 
 * @return the play address
 */ 
int LibSidplay2_impl::getPlayAddr() const {
    return pSidInfo->playAddr;
}

/**
 * Get the number of songs
 * 
 * @return the number of songs
 */ 
int LibSidplay2_impl::getSongs() const {
    return pSidInfo->songs;
}

/**
 * Get the current song
 * 
 * @return the current song
 */ 
int LibSidplay2_impl::getCurrentSong() const {
    return pSidInfo->currentSong;
}

/**
 * Get thwe number of starting song
 * 
 * @return the starting song
 */ 
int LibSidplay2_impl::getStartSong() const {
    return pSidInfo->startSong;
}

/**
 * Get the info string of the given index
 * 
 * @param i the index
 * @return the info string
 */ 
const char* LibSidplay2_impl::getInfoString(int i) const {
    return pSidInfo->infoString[i];
}

/**
 * Get numebr of info string
 * 
 * @return the number of imput string
 */ 
int LibSidplay2_impl::getInfoStringsNum() const {
    return pSidInfo->numberOfInfoStrings;
}

/**
 * Get the status string
 * 
 * @return the status string
 */ 
const char* LibSidplay2_impl::getStatusString() const {
    return pSidInfo->statusString;
}

/**
 * Get the speed string
 * 
 * @return the speed string
 */
const char* LibSidplay2_impl::getSpeedString2() const {
    return pSidInfo->speedString;
}

/**
 * Get the format string
 * 
 * @return the formal string
 */
const char* LibSidplay2_impl::getFormatString() const {    
  return pSidInfo->formatString;
}

/**
 * Get status
 * 
 * @return the status
 */ 
bool LibSidplay2_impl::getStatus() {
  return SidTune::getStatus();
}

/**
 * Get MD5
 * 
 * @return the MD5
 */    
const QString& LibSidplay2_impl::getMD5_Digest() {
    MD5 myMD5;
    createMD5(myMD5);
    myMD5.finish();
    // Print fingerprint.
    QString tmp;
    pDigest->truncate(0);
    for (int di = 0; di < 16; ++di)
      *pDigest += tmp.sprintf("%02x",(int)myMD5.getDigest()[di]);
    return *pDigest;  
}

/**
 * Create MD5
 * 
 * @param myMD5 MD5 to create
 */
void LibSidplay2_impl::createMD5(MD5& myMD5) {
  if (SidTune::status) {
    // Include C64 data.

    myMD5.append(SidTune::cache.get()+SidTune::fileOffset,SidTune::info.c64dataLen);
    unsigned char tmp[2];

    // Include INIT and PLAY address.
    writeLEword(tmp,SidTune::info.initAddr);
    myMD5.append(tmp,sizeof(tmp));
    writeLEword(tmp,SidTune::info.playAddr);
    myMD5.append(tmp,sizeof(tmp));
    // Include number of songs.
    writeLEword(tmp,SidTune::info.songs);
    myMD5.append(tmp,sizeof(tmp));
    // Include song speed for each song.
    for (unsigned int s = 1; s <= SidTune::info.songs; s++) {
      SidTune::selectSong(s);
      myMD5.append(&SidTune::info.songSpeed,sizeof(SidTune::info.songSpeed));
    }
    // Deal with PSID v2NG clock speed flags: Let only NTSC
    // clock speed change the MD5 fingerprint. That way the
    // fingerprint of a PAL-speed sidtune in PSID v1, v2, and
    // PSID v2NG format is the same.
    if ( SidTune::info.clockSpeed == SIDTUNE_CLOCK_NTSC )
      myMD5.append(&SidTune::info.clockSpeed,sizeof(SidTune::info.clockSpeed));

     // NB! If the fingerprint is used as an index into a
     // song-lengths database or cache, modify above code to
     // allow for PSID v2NG files which have clock speed set to
     // SIDTUNE_CLOCK_ANY. If the SID player program fully
     // supports the SIDTUNE_CLOCK_ANY setting, a sidtune could
     // either create two different fingerprints depending on
     // the clock speed chosen by the player, or there could be
     // two different values stored in the database/cache.
  }  
}

/**
 * Get the data buffer
 * 
 * @return the data buffer
 */ 
const unsigned char *LibSidplay2_impl::getDataBuffer() {
  return SidTune::cache.get();
}

/**
 * Get the data length
 * 
 * @return the data length
 */
int LibSidplay2_impl::getDataLength() {
  return SidTune::cache.len();
}




LibSidplay2::LibSidplay2(): pimpl(new LibSidplay2_impl) {
}

LibSidplay2::~LibSidplay2() {
}

const EmuConfig& LibSidplay2::getConfig() { 
  return pimpl->getConfig(); 
  
}

const EmuConfig& LibSidplay2::getConfigDefault() {
  return pimpl->getConfigDefault();   
}

void LibSidplay2::setConfig(const EmuConfig& inConfig) {
  pimpl->setConfig(inConfig);  
} 

void LibSidplay2::initSong(int song) {
  pimpl->initSong(song);
} 

void LibSidplay2::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
  pimpl->fillBuffer(pBuffer, bufferSize);
}

void LibSidplay2::stop() {
  pimpl->stop();
}

void LibSidplay2::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
  pimpl->setVoiceVolume(voice, left, right, total);
}

void LibSidplay2::resetSecondsThisSong() {
  pimpl->resetSecondsThisSong();
}

int LibSidplay2::getSecondsThisSong() {
  return pimpl->getSecondsThisSong();
}

int LibSidplay2::getSecondsTotal() {
  return pimpl->getSecondsTotal();
}

const char* LibSidplay2::getVersionString() {
  return pimpl->getVersionString();
}

const char* LibSidplay2::getSpeedString() {
  return pimpl->getSpeedString();
}

const char* LibSidplay2::getSidModelString() {
  return pimpl->getSidModelString();
}

const char* LibSidplay2::getClockFreqString() {
  return pimpl->getClockFreqString();
}

int LibSidplay2::getCia1TA() {
  return pimpl->getCia1TA();
}

int LibSidplay2::getClockSpeed() {
  return pimpl->getClockSpeed();
}

void LibSidplay2::createMD5(MD5& myMD5) {
  pimpl->createMD5(myMD5);
}

const unsigned char *LibSidplay2::getDataBuffer() {
  return pimpl->getDataBuffer(); 
}

int LibSidplay2::getDataLength() {
  return pimpl->getDataLength();
}

int LibSidplay2::getMaxSongs() const {
  return pimpl->getMaxSongs();
}

unsigned long int LibSidplay2::getMaxSidFileLen() const {
  return pimpl->getMaxSidFileLen();
}

bool LibSidplay2::open(const QString& fileName) {
  return pimpl->open(fileName);
}

bool LibSidplay2::load(void* buf, unsigned long int bufLen) {
  return pimpl->load(buf, bufLen);
}

bool LibSidplay2::savePSID(const QString& fileName, bool overWrite) {
 return pimpl->savePSID(fileName, overWrite);
}

void LibSidplay2::updateInfo() {
 return pimpl->updateInfo();
}

int LibSidplay2::getLoadAddr() const {
  return pimpl->getLoadAddr();
}

int LibSidplay2::getInitAddr() const {
  return pimpl->getInitAddr();
}

int LibSidplay2::getPlayAddr() const {
  return pimpl->getPlayAddr();
}

int LibSidplay2::getSongs() const {
  return pimpl->getSongs();
}

int LibSidplay2::getCurrentSong() const {
  return pimpl->getCurrentSong();
}

int LibSidplay2::getStartSong() const {
  return pimpl->getStartSong();
}

const char* LibSidplay2::getInfoString(int i) const {
  return pimpl->getInfoString(i);
}

int LibSidplay2::getInfoStringsNum() const {
  return pimpl->getInfoStringsNum();
}

const char* LibSidplay2::getStatusString() const {
  return pimpl->getStatusString();
}

const char* LibSidplay2::getSpeedString2() const {
 return pimpl->getSpeedString2(); 
}

const char* LibSidplay2::getFormatString() const {
 return pimpl->getFormatString();
}

bool LibSidplay2::getStatus() {
  return pimpl->getStatus();
}

const QString& LibSidplay2::getMD5_Digest() {
  return pimpl->getMD5_Digest();
}


#endif  /* LIBSIDPLAY2 */