/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LIBSIDPLAYFP
#define LIBSIDPLAYFP

#include <sidplayfp/sidplay2.h>
#include <sidplayfp/builders/resid.h>
#include <sidplayfp/builders/resid-emu.h>
#include <sidplayfp/SidTune.h>
#include "../emucfg3.h"
#include "../MD5.h"
#include "LibSidplayFp.h"

/**
 * Manage LibSidplayFp library
 */ 
class LibSidplayFp_impl : public SidLibrary, public  SidTune {
  protected:
    sidplay2* pEmuEngine;
    ReSIDBuilder* pEmuSid;
    sid2_config_t sid2_cfg;
    sid2_extra_t sid2_extra;
    SidTuneInfo* pSidInfo;
    QString* pDigest;
    
  private:
    inline void writeLEword(unsigned char ptr[2], int someWord) {
      ptr[0] = (someWord & 0xFF);
      ptr[1] = (someWord >> 8);
    }
    
  public:
    
    /**
     * Construct the library 
     */
    LibSidplayFp_impl();
    
    /**
     * Deconstruct the library
     */ 
    ~LibSidplayFp_impl();
    
    /**
     * Get configuration
     * 
     * @return the configuration
     */ 
    const EmuConfig& getConfig();
    
    /**
     * Get default configuration
     * 
     * @return the default configuration
     */
    const EmuConfig& getConfigDefault();   
    
    /**
     * Set the configuration to use
     * 
     * @param inConfig input configuration
     */ 
    void setConfig(const EmuConfig& inConfig);     
    
    /**
     * Init with the given song
     * 
     * @param song the given number of song to init
     */ 
    void initSong(int song);  
    
    /**
     * Fill the buffer with played music
     * 
     * @param pBuffer the buffer to fill
     * @param bufferSize the size of the buffer
     */ 
    void fillBuffer(void* pBuffer, unsigned long int bufferSize);    
    
    /**
     * Stop the engine
     */
    void stop();   
    
    /**
     * Set the volume for the given voice
     * 
     * @param voice the number of voice
     * @param left left volume
     * @param right right volume
     * @param total total volume
     */
    void setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total);
    
    /**
     * Reset the seconds of this song
     */
    void resetSecondsThisSong();  
         
    /**
     * Get the seconds of this song
     * 
     * @return the seconds of this song
     */    
    int getSecondsThisSong();   
    
    /**
     * Get the total seconds
     * 
     * @return the total seconds
     */    
    int getSecondsTotal();    
    
    /**
     * Get string version
     * 
     * @return the string version
     */
    const char* getVersionString();     
    
    /**
     * Get speed string
     * 
     * @return the speed string
     */
    const char* getSpeedString();    
    
    /**
     * Get sid model string
     * 
     * @return the sid model string
     */    
    const char* getSidModelString();  
        
    /**
     * Get clock frequnecy string
     * 
     * @return the clock frequency string
     */      
    const char* getClockFreqString();      
    
    /**
     * Get last CIA1 TA value of C64
     * 
     * @return the last CIA1 TA value
     */    
    int getCia1TA();  
    
    /**
     * Get clock speed     
     *  
     * @return the clock speed (-1=not implemented)
     */
    int getClockSpeed();    
    
    /**
     * Get the name of Sid Library managed
     * 
     * @return the name of sid library managed
     */
    const char* getName() const {
      return LIB_LFP;
    }    
    
    /**
     * Get the enum of Sid Library managed
     * 
     * @return the enum of sid library managed
     */
    const int getEnum() const {
      return EmuConfig::LFP;
    }    
    
    
    /**
     * Get the max songs
     * 
     * @return the max songs
     */ 
    int getMaxSongs() const; 
    
    /**
     * Get the max sid file length
     * 
     * @return the max sid file length
     */ 
    unsigned long int getMaxSidFileLen() const;  
        
    /**
     * Open the give file
     * 
     * @param fileName the name of file
     * @return true if operation is ok 
     */ 
    bool open(const QString& fileName);    
            
    /**
     * Load the file in the given buffer
     * 
     * @param buf the buffer
     * @param bufLen the length of buffer
     * @return true if the operation is ok
     */ 
    bool load(void*, unsigned long int);  
    
    /**
     * Save the file as PSID
     * 
     * @param fileName the fileName to used
     * @param overWrite true if over write
     */ 
    bool savePSID(const QString&, bool);   
    
    /**
     * Update information
     */ 
    void updateInfo();   
    
   /**
    * Get the load address
    * 
    * @return the load address
    */ 
   int getLoadAddr() const;

   /**
    * Get the init address
    * 
    * @return the init address
    */
   int getInitAddr() const;

   /**
    * Get the play address
    * 
    * @return the play address
    */ 
   int getPlayAddr() const;

   /**
    * Get the number of songs
    * 
    * @return the number of songs
    */ 
   int getSongs() const;

   /**
    * Get the current song
    * 
    * @return the current song
    */ 
   int getCurrentSong() const;

   /**
    * Get thwe number of starting song
    *  
    * @return the starting song
    */ 
   int getStartSong() const;

   /**
    * Get the info string of the given index
    * 
    * @param i the index
    * @return the info string
    */ 
   const char* getInfoString(int i) const;

   /**
    * Get numebr of info string
    * 
    * @return the number of imput string
    */ 
   int getInfoStringsNum() const;

   /**
    * Get the status string
    * 
    * @return the status string
    */ 
   const char* getStatusString() const;

   /**
    * Get the speed string
    * 
    * @return the speed string
    */
   const char* getSpeedString2() const;

   /**
    * Get the format string
    * 
    * @return the formal string
    */ 
   const char* getFormatString() const;
   
   /**
    * Get status
    * 
    * @return the status
    */ 
   bool getStatus();

   /**
    * Get MD5
    * 
    * @return the MD5
    */    
   const QString& getMD5_Digest();   
   
    /**
     * Create MD5
     * 
     * @param myMD5 MD5 to create
     */
    void createMD5(MD5& myMD5);
    
   /**
    * Get the data buffer
    * 
    * @return the data buffer
    */ 
   const unsigned char *getDataBuffer();

   /**
    * Get the data length
    * 
    * @return the data length
    */
   int getDataLength();   
   
protected:
  /**
   * Init the resid engine with the actual configuration
   */
  bool resid_init();
    
   /**
    * Init the given resid object
    * 
    * @param sid the sid to use
    * @param speed speed to use
    * @param cycles_per_sec cycle per sec to use
    * @return true if operation is ok
    */
   bool resid_init(ReSID *sid, int speed, int cycles_per_sec);
};

/**
 * Construct the library
 */ 
LibSidplayFp_impl::LibSidplayFp_impl():SidTune(0)  {
  pEmuEngine = new sidplay2;
  pEmuSid = new ReSIDBuilder("XSIDPLAY reSID");
  sid2_cfg = pEmuEngine->config();
  sid2_cfg.sidSamples = false;
  myEmuConfig.pConfig3->emulateFilter_Re = true;
  if (*pEmuSid) {  // TODO: nothrow above
    // Builder initialisation ok.
    pEmuSid->create( (pEmuEngine->info()).maxsids );
    sid2_cfg.sidEmulation = pEmuSid;
    pEmuEngine->config(sid2_cfg);
  }
  myEmuConfigDefault = getConfig ();
  
  pSidInfo = new SidTuneInfo;
  pDigest = new QString;
}

/**
 * Deconstruct the library
 */ 
LibSidplayFp_impl::~LibSidplayFp_impl() {
  delete pEmuSid;
  delete pEmuEngine;
  delete pSidInfo;
  delete pDigest;
}

/**
 * Get configuration
 * 
 * @return the configuration
 */ 
const EmuConfig& LibSidplayFp_impl::getConfig() {
  // Convert config to sidplay1.
  sid2_cfg = pEmuEngine->config();
  myEmuConfig.pConfig3->frequency  = (int) sid2_cfg.frequency;
///  myEmuConfig.pConfig3->bitsPerSample = sid2_cfg.precision;
  myEmuConfig.pConfig3->sampleFormat = SIDEMU_SIGNED_PCM_L3;
 /// if (sid2_cfg.precision == 8)
 ///   myEmuConfig.pConfig3->sampleFormat = SIDEMU_UNSIGNED_PCM_L3;
  myEmuConfig.pConfig3->channels = SIDEMU_MONO_L3;
  if (sid2_cfg.playback == sid2_stereo)
    myEmuConfig.pConfig3->channels = SIDEMU_STEREO_L3;
    

  // resid
  myEmuConfig.pConfig3->forceSidModel_Re = ( sid2_cfg.sidModel != SID2_MODEL_CORRECT );    
  myEmuConfig.pConfig3->mos8580_Re = ( sid2_cfg.sidDefault == SID2_MOS8580 );
  myEmuConfig.pConfig3->digibooster_Re = sid2_extra.digibooster;  
  myEmuConfig.pConfig3->clockSpeed_Re = SIDTUNE_CLOCK_PAL;
  if (sid2_cfg.clockDefault == SID2_CLOCK_NTSC)
    myEmuConfig.pConfig3->clockSpeed_Re = SIDTUNE_CLOCK_NTSC;  
  myEmuConfig.pConfig3->forceSongSpeed_Re = sid2_cfg.clockForced;    
    
  switch (sid2_cfg.environment) {
    case sid2_envPS:
      myEmuConfig.pConfig3->memoryMode_Re = MPU_PLAYSID_ENVIRONMENT_L3;
      break;
    case sid2_envTP:
      myEmuConfig.pConfig3->memoryMode_Re = MPU_TRANSPARENT_ROM_L3;
      break;
    case sid2_envBS:
      myEmuConfig.pConfig3->memoryMode_Re = MPU_BANK_SWITCHING_L3;
      break;      
    case sid2_envR:
    default:
      myEmuConfig.pConfig3->memoryMode_Re = MPU_REALSID_ENVIRONMENT_L3;
      break;
  }

  return myEmuConfig; 
}
    
/**
 * Get default configuration
 * 
 * @return the default configuration
 */
const EmuConfig& LibSidplayFp_impl::getConfigDefault() {
  return myEmuConfigDefault;  
}

/**
 * Set the configuration to use
 * 
 * @param inConfig input configuration
 */ 
void LibSidplayFp_impl::setConfig(const EmuConfig& inConfig) {
    // make a copy of it
    myEmuConfig=inConfig;
    
    // Convert config for sidplayfp
    sid2_cfg.frequency = (uint_least32_t) inConfig.pConfig3->frequency;
    ///sid2_cfg.precision = inConfig.pConfig3->bitsPerSample;
    sid2_cfg.playback = sid2_mono;
    if (inConfig.pConfig3->channels == SIDEMU_STEREO_L3)
        sid2_cfg.playback = sid2_stereo;
    sid2_cfg.sidModel = SID2_MODEL_CORRECT;

    sid2_cfg.sidSamples = true;
    sid2_cfg.clockSpeed = SID2_CLOCK_CORRECT;
    
    ///pEmuSid->sampling (sid2_cfg.frequency);
    
    
    // resid
  
    sid2_extra.digibooster = inConfig.pConfig3->digibooster_Re;
    if ( inConfig.pConfig3->mos8580_Re )
        sid2_cfg.sidDefault = SID2_MOS8580;
    else
        sid2_cfg.sidDefault = SID2_MOS6581;    
    if ( inConfig.pConfig3->forceSidModel_Re )
        sid2_cfg.sidModel = sid2_cfg.sidDefault;         
    if (inConfig.pConfig3->clockSpeed_Re == SIDTUNE_CLOCK_NTSC)
        sid2_cfg.clockDefault = SID2_CLOCK_NTSC;
    else
        sid2_cfg.clockDefault = SID2_CLOCK_PAL;
    if ( sid2_cfg.clockForced = inConfig.pConfig3->forceSongSpeed_Re )
        sid2_cfg.clockSpeed = sid2_cfg.clockDefault;    

    pEmuSid->filter(inConfig.pConfig3->emulateFilter_Re);
    
    switch (inConfig.pConfig3->memoryMode_Re)
    {
    case MPU_PLAYSID_ENVIRONMENT_L3:
        sid2_cfg.environment = sid2_envPS;
        break;
    case MPU_TRANSPARENT_ROM_L3:
        sid2_cfg.environment = sid2_envTP;
        break;
    case MPU_BANK_SWITCHING_L3:
        sid2_cfg.environment = sid2_envBS;
        break;
    case MPU_REALSID_ENVIRONMENT_L3:
    default:
        sid2_cfg.environment = sid2_envR;
        break;
    }

    pEmuEngine->config(sid2_cfg);
    sid2_cfg = pEmuEngine->config();  
    
    // initialize resid;
    resid_init();
}

/**
 * Init with the given song
 * 
 * @param song the given number of song to init
 */ 
void LibSidplayFp_impl::initSong(int song) {
  SidTune::selectSong(song);
  pEmuEngine->load(this);
}

/**
 * Fill the buffer with played music
 * 
 * @param pBuffer the buffer to fill
 * @param bufferSize the size of the buffer
 */ 
void LibSidplayFp_impl::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
  // compensate buffer length for different size
  pEmuEngine->play((short int*)pBuffer,
                   (uint_least32_t)(bufferSize*sizeof(unsigned char)/sizeof(short)));
}

/**
 * Set the volume for the given voice
 * 
 * @param voice the number of voice
 * @param left left volume
 * @param right right volume
 * @param total total volume
 */
void LibSidplayFp_impl::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
}  

/**
 * Reset the seconds of this song
 */
void LibSidplayFp_impl::resetSecondsThisSong() {  
}

/**
 * Get the seconds of this song
 * 
 * @return the seconds of this song
 */    
int LibSidplayFp_impl::getSecondsThisSong() {
  return pEmuEngine->time() / pEmuEngine->timebase();
}
 
/**
 * Get the total seconds
 * 
 * @return the total seconds
 */    
int LibSidplayFp_impl::getSecondsTotal() {
  return pEmuEngine->mileage() / pEmuEngine->timebase();
}      

/**
 * Get string version
 * 
 * @return the string version
 */
const char* LibSidplayFp_impl::getVersionString() {
  sid2_info_t info;
  info = pEmuEngine->info();
  
  versionString = "SID player library   Version ";
  versionString += info.version;
  versionString += pEmuSid->credits();
  
  return versionString.toLatin1();
}

/**
 * Get speed string
 * 
 * @return the speed string
 */
const char* LibSidplayFp_impl::getSpeedString() {
  sid2_info_t info = pEmuEngine->info();
  return info.tuneInfo->speedString;
}

/**
 * Get sid model string
 * 
 * @return the sid model string
 */    
const char* LibSidplayFp_impl::getSidModelString() {
  sid2_info_t info = pEmuEngine->info();

  if ( info.tuneInfo->sidModel1 == SID2_MOS8580 ) return "MOS 8580";
  else return "MOS 6581";
}

/**
 * Get clock frequnecy string
 * 
 * @return the clock frequency string
 */      
const char* LibSidplayFp_impl::getClockFreqString() {
  sid2_info_t info = pEmuEngine->info();

  switch (info.tuneInfo->clockSpeed) {
    case SIDTUNE_CLOCK_UNKNOWN:
      return "Unknown";
    case SIDTUNE_CLOCK_PAL:
      return "PAL";
    case SIDTUNE_CLOCK_NTSC:
      return "NTSC";
    case SIDTUNE_CLOCK_ANY:
      return "ANY";
  }
  return "";  
} 
 
/**
 * Stop the engine
 */
void LibSidplayFp_impl::stop() {
  pEmuEngine->stop();
}  

/**
 * Get last CIA1 TA value of C64
 * 
 * @return the last CIA1 TA value
 */    
int LibSidplayFp_impl::getCia1TA() {
  return pEmuEngine->cia1_last_ta();
}

/**
 * Get clock speed     
 *  
 * @return the clock speed (-1=not implemented)
 */
int LibSidplayFp_impl::getClockSpeed()
{
  sid2_info_t info = pEmuEngine->info();
  return info.tuneInfo->clockSpeed;
}



/**
 * Get the max songs
 * 
 * @return the max songs
 */ 
int LibSidplayFp_impl::getMaxSongs() const {
  return SIDTUNE_MAX_SONGS;
}

/**
 * Get the max sid file length
 * 
 * @return the max sid file length
 */ 
unsigned long int LibSidplayFp_impl::getMaxSidFileLen() const {
  return SIDTUNE_MAX_FILELEN;
}

/**
 * Open the give file
 * 
 * @param fileName the name of file
 * @return true if operation is ok 
 */ 
bool LibSidplayFp_impl::open(const QString& fileName) {
  bool ret = SidTune::load(fileName.toLatin1());
  updateInfo();
  return ret;  
}

/**
 * Load the file in the given buffer
 * 
 * @param buf the buffer
 * @param bufLen the length of buffer
 * @return true if the operation is ok
 */ 
bool LibSidplayFp_impl::load(void* buf, unsigned long int bufLen) {
  bool ret =  SidTune::read((const ubyte_emuwt*)buf,(udword_emuwt)bufLen);
  updateInfo();
  return ret;  
}

/**
 * Save the file as PSID
 * 
 * @param fileName the fileName to used
 * @param overWrite true if over write
 */ 
bool LibSidplayFp_impl::savePSID(const QString& fileName, bool overWrite) {
  return SidTune::savePSIDfile(fileName.toLatin1(),overWrite);
}

/**
 * Update information
 */ 
void LibSidplayFp_impl::updateInfo() {
  SidTune::getInfo(*pSidInfo); 
}

/**
 * Get the load address
 * 
 * @return the load address
 */ 
int LibSidplayFp_impl::getLoadAddr() const {
  return pSidInfo->loadAddr;
}

/**
 * Get the init address
 * 
 * @return the init address
 */
int LibSidplayFp_impl::getInitAddr() const {
    return pSidInfo->initAddr;
}

/**
 * Get the play address
 * 
 * @return the play address
 */ 
int LibSidplayFp_impl::getPlayAddr() const {
    return pSidInfo->playAddr;
}

/**
 * Get the number of songs
 * 
 * @return the number of songs
 */ 
int LibSidplayFp_impl::getSongs() const {
    return pSidInfo->songs;
}

/**
 * Get the current song
 * 
 * @return the current song
 */ 
int LibSidplayFp_impl::getCurrentSong() const {
    return pSidInfo->currentSong;
}

/**
 * Get thwe number of starting song
 * 
 * @return the starting song
 */ 
int LibSidplayFp_impl::getStartSong() const {
    return pSidInfo->startSong;
}

/**
 * Get the info string of the given index
 * 
 * @param i the index
 * @return the info string
 */ 
const char* LibSidplayFp_impl::getInfoString(int i) const {
    return pSidInfo->infoString[i];
}

/**
 * Get numebr of info string
 * 
 * @return the number of imput string
 */ 
int LibSidplayFp_impl::getInfoStringsNum() const {
    return pSidInfo->numberOfInfoStrings;
}

/**
 * Get the status string
 * 
 * @return the status string
 */ 
const char* LibSidplayFp_impl::getStatusString() const {
    return pSidInfo->statusString;
}

/**
 * Get the speed string
 * 
 * @return the speed string
 */
const char* LibSidplayFp_impl::getSpeedString2() const {
    return pSidInfo->speedString;
}

/**
 * Get the format string
 * 
 * @return the formal string
 */
const char* LibSidplayFp_impl::getFormatString() const {    
  return pSidInfo->formatString;
}

/**
 * Get status
 * 
 * @return the status
 */ 
bool LibSidplayFp_impl::getStatus() {
  return SidTune::getStatus();
}

/**
 * Get MD5
 * 
 * @return the MD5
 */    
const QString& LibSidplayFp_impl::getMD5_Digest() {
    MD5 myMD5;
    createMD5(myMD5);
    myMD5.finish();
    // Print fingerprint.
    QString tmp;
    pDigest->truncate(0);
    for (int di = 0; di < 16; ++di)
      *pDigest += tmp.sprintf("%02x",(int)myMD5.getDigest()[di]);
    return *pDigest;  
}

/**
 * Create MD5
 * 
 * @param myMD5 MD5 to create
 */
void LibSidplayFp_impl::createMD5(MD5& myMD5) {
  if (SidTune::status) {
    // Include C64 data.

    myMD5.append(SidTune::cache.get()+SidTune::fileOffset,SidTune::info.c64dataLen);
    unsigned char tmp[2];

    // Include INIT and PLAY address.
    writeLEword(tmp,SidTune::info.initAddr);
    myMD5.append(tmp,sizeof(tmp));
    writeLEword(tmp,SidTune::info.playAddr);
    myMD5.append(tmp,sizeof(tmp));
    // Include number of songs.
    writeLEword(tmp,SidTune::info.songs);
    myMD5.append(tmp,sizeof(tmp));
    // Include song speed for each song.
    for (unsigned int s = 1; s <= SidTune::info.songs; s++) {
      SidTune::selectSong(s);
      myMD5.append(&SidTune::info.songSpeed,sizeof(SidTune::info.songSpeed));
    }
    // Deal with PSID v2NG clock speed flags: Let only NTSC
    // clock speed change the MD5 fingerprint. That way the
    // fingerprint of a PAL-speed sidtune in PSID v1, v2, and
    // PSID v2NG format is the same.
    if ( SidTune::info.clockSpeed == SIDTUNE_CLOCK_NTSC )
      myMD5.append(&SidTune::info.clockSpeed,sizeof(SidTune::info.clockSpeed));

     // NB! If the fingerprint is used as an index into a
     // song-lengths database or cache, modify above code to
     // allow for PSID v2NG files which have clock speed set to
     // SIDTUNE_CLOCK_ANY. If the SID player program fully
     // supports the SIDTUNE_CLOCK_ANY setting, a sidtune could
     // either create two different fingerprints depending on
     // the clock speed chosen by the player, or there could be
     // two different values stored in the database/cache.
  }  
}

/**
 * Get the data buffer
 * 
 * @return the data buffer
 */ 
const unsigned char *LibSidplayFp_impl::getDataBuffer() {
  return SidTune::cache.get();
}

/**
 * Get the data length
 * 
 * @return the data length
 */
int LibSidplayFp_impl::getDataLength() {
  return SidTune::cache.len();
}

/**
 * Init the given resid object (from vice emu)
 * 
 * @param sid the sid to use
 * @param speed speed to use
 * @param cycles_per_sec cycle per sec to use
 * @return true if operation is ok
 */
bool LibSidplayFp_impl::resid_init(ReSID *sid, int speed, int cycles_per_sec) {
    reSID::sampling_method method;
    double passband, gain;
    int filters_enabled, model, sampling, passband_percentage, gain_percentage, filter_bias_mV;

    filters_enabled=myEmuConfig.pConfig3->emulateFilter_Re;
    model=myEmuConfig.pConfig3->sidChips;
    sampling=myEmuConfig.pConfig3->sampling_Re;
    passband_percentage=myEmuConfig.pConfig3->passbandPerc_Re;
    gain_percentage=myEmuConfig.pConfig3->filterGainPerc_Re;
    filter_bias_mV=0;

    passband = speed * passband_percentage / 200.0;
    gain = gain_percentage / 100.0;

    switch (model) {
      default:
      case 0: // "MOS6581"
        sid->direct_set_chip_model(reSID::MOS6581);
        sid->direct_set_voice_mask(0x07);
        sid->direct_input(0);
        break;
      case 1: // MOS8580
        sid->direct_set_chip_model(reSID::MOS8580);
        sid->direct_set_voice_mask(0x07);
        sid->direct_input(0);
        break;
      case 2: // MOS8580 + digi boost
        sid->direct_set_chip_model(reSID::MOS8580);
        sid->direct_set_voice_mask(0x0f);
        sid->direct_input(-32768);
        break;
    }
    sid->direct_enable_filter(filters_enabled ? true : false);
    sid->direct_adjust_filter_bias(filter_bias_mV / 1000.0);
    sid->direct_enable_external_filter(filters_enabled ? true : false);

    switch (sampling) {
      default:
      case 0:
        method = reSID::SAMPLE_FAST;
        break;
      case 1:
        method = reSID::SAMPLE_INTERPOLATE;
        break;
      case 2:
        method = reSID::SAMPLE_RESAMPLE;
        break;
      case 3:
        method = reSID::SAMPLE_RESAMPLE_FASTMEM;
        break;
    }        

    if (!sid->direct_set_sampling_parameters(cycles_per_sec, method,
                                            speed, passband, gain)) {        
        return false;
    }  
  return true;
}

/**
 * Init the resid engine with the actual configuration
 * 
 * @return true if operation is ok
 */
bool LibSidplayFp_impl::resid_init() {
  bool result=true;
  
  const int size = pEmuSid->sidobjs.size();
  
  // determine cycles_per_sec
  int cycles_per_sec=985248;  // suppose PAL in every case
  
  // SidTune::info.clockSpeed
  
  // use speed of song?
  if (!myEmuConfig.pConfig3->forceSongSpeed_Re) {
    if (SidTune::info.clockSpeed==SID2_CLOCK_PAL) cycles_per_sec=985248;
    if (SidTune::info.clockSpeed==SID2_CLOCK_NTSC) cycles_per_sec=1022730;
  } else {
      // use forced speed
      if (myEmuConfig.pConfig3->clockSpeed_Re==SID2_CLOCK_PAL) cycles_per_sec=985248;
      if (myEmuConfig.pConfig3->clockSpeed_Re==SID2_CLOCK_NTSC) cycles_per_sec=1022730;
    }
  
  for (int i = 0; i < size; i++) {
    ReSID *sid = (ReSID *) pEmuSid->sidobjs[i];
    result &= resid_init(sid, myEmuConfig.pConfig3->frequency, cycles_per_sec);
  }  
  
  return result;
}



LibSidplayFp::LibSidplayFp(): pimpl(new LibSidplayFp_impl) {
}

LibSidplayFp::~LibSidplayFp() {
}

const EmuConfig& LibSidplayFp::getConfig() { 
  return pimpl->getConfig(); 
  
}

const EmuConfig& LibSidplayFp::getConfigDefault() {
  return pimpl->getConfigDefault();   
}

void LibSidplayFp::setConfig(const EmuConfig& inConfig) {
  pimpl->setConfig(inConfig);  
} 

void LibSidplayFp::initSong(int song) {
  pimpl->initSong(song);
} 

void LibSidplayFp::fillBuffer(void* pBuffer, unsigned long int bufferSize) {
  pimpl->fillBuffer(pBuffer, bufferSize);
}

void LibSidplayFp::stop() {
  pimpl->stop();
}

void LibSidplayFp::setVoiceVolume(int voice, ubyte_emuwt left, ubyte_emuwt right, uword_emuwt total) {
  pimpl->setVoiceVolume(voice, left, right, total);
}

void LibSidplayFp::resetSecondsThisSong() {
  pimpl->resetSecondsThisSong();
}

int LibSidplayFp::getSecondsThisSong() {
  return pimpl->getSecondsThisSong();
}

int LibSidplayFp::getSecondsTotal() {
  return pimpl->getSecondsTotal();
}

const char* LibSidplayFp::getVersionString() {
  return pimpl->getVersionString();
}

const char* LibSidplayFp::getSpeedString() {
  return pimpl->getSpeedString();
}

const char* LibSidplayFp::getSidModelString() {
  return pimpl->getSidModelString();
}

const char* LibSidplayFp::getClockFreqString() {
  return pimpl->getClockFreqString();
}

int LibSidplayFp::getCia1TA() {
  return pimpl->getCia1TA();
}

int LibSidplayFp::getClockSpeed() {
  return pimpl->getClockSpeed();
}

void LibSidplayFp::createMD5(MD5& myMD5) {
  pimpl->createMD5(myMD5);
}

const unsigned char *LibSidplayFp::getDataBuffer() {
  return pimpl->getDataBuffer(); 
}

int LibSidplayFp::getDataLength() {
  return pimpl->getDataLength();
}

int LibSidplayFp::getMaxSongs() const {
  return pimpl->getMaxSongs();
}

unsigned long int LibSidplayFp::getMaxSidFileLen() const {
  return pimpl->getMaxSidFileLen();
}

bool LibSidplayFp::open(const QString& fileName) {
  return pimpl->open(fileName);
}

bool LibSidplayFp::load(void* buf, unsigned long int bufLen) {
  return pimpl->load(buf, bufLen);
}

bool LibSidplayFp::savePSID(const QString& fileName, bool overWrite) {
 return pimpl->savePSID(fileName, overWrite);
}

void LibSidplayFp::updateInfo() {
 return pimpl->updateInfo();
}

int LibSidplayFp::getLoadAddr() const {
  return pimpl->getLoadAddr();
}

int LibSidplayFp::getInitAddr() const {
  return pimpl->getInitAddr();
}

int LibSidplayFp::getPlayAddr() const {
  return pimpl->getPlayAddr();
}

int LibSidplayFp::getSongs() const {
  return pimpl->getSongs();
}

int LibSidplayFp::getCurrentSong() const {
  return pimpl->getCurrentSong();
}

int LibSidplayFp::getStartSong() const {
  return pimpl->getStartSong();
}

const char* LibSidplayFp::getInfoString(int i) const {
  return pimpl->getInfoString(i);
}

int LibSidplayFp::getInfoStringsNum() const {
  return pimpl->getInfoStringsNum();
}

const char* LibSidplayFp::getStatusString() const {
  return pimpl->getStatusString();
}

const char* LibSidplayFp::getSpeedString2() const {
 return pimpl->getSpeedString2(); 
}

const char* LibSidplayFp::getFormatString() const {
 return pimpl->getFormatString();
}

bool LibSidplayFp::getStatus() {
  return pimpl->getStatus();
}

const QString& LibSidplayFp::getMD5_Digest() {
  return pimpl->getMD5_Digest();
}


#endif  /* LIBSIDPLAYFP */