/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef SIDLIBRARY_H
#define SIDLIBRARY_H

#include "EmuConfig.h"
#include <qstring.h>
#include "../TypeWrapper.h"

class MD5;

/**
 * Manage a Sid Library used by the various wrapper
 */
class SidLibrary {
  protected:  // ---------------------------------------------------- protected

    const char* errorString;
    QString versionString;
    EmuConfig myEmuConfig;
    EmuConfig myEmuConfigDefault;
    
  public:  // ---------------------------------------------------------- public
  
    SidLibrary() { errorString = "No Sid library."; }
    virtual ~SidLibrary() { ; }
    
    // thread-safe of EmuWrapper
    
    
    /**
     * Get configuration
     * 
     * @return the configuration
     */ 
    virtual const EmuConfig& getConfig() = 0;
    
    /**
     * Get default configuration
     * 
     * @return the default configuration
     */
    virtual const EmuConfig& getConfigDefault() = 0;
    
    /**
     * Set the configuration to use
     * 
     * @param inConfig input configurationà
     */ 
    virtual void setConfig(const EmuConfig&) = 0;
    
    /**
     * Init with the given song
     * 
     * @param song the given number of song to init
     */ 
    virtual void initSong(int) = 0;
    
    /**
     * Fill the buffer with played music
     * 
     * @param pBuffer the buffer to fill
     * @param bufferSize the size of the buffer
     */ 
    virtual void fillBuffer(void* pBuffer, unsigned long int bufferSize) = 0;
    
    /**
     * Stop the engine
     */
    virtual void stop() = 0;
    
    /**
     * Set the volume for the given voice
     * 
     * @param voice the number of voice
     * @param left left volume
     * @param right right volume
     * @param total total volume
     */
    virtual void setVoiceVolume(int, ubyte_emuwt, ubyte_emuwt, uword_emuwt) = 0;
    
    /**
     * Reset the seconds of this song
     */
    virtual void resetSecondsThisSong() = 0;
    
    /**
     * Get the seconds of this song
     * 
     * @return the seconds of this song
     */    
    virtual int getSecondsThisSong() = 0;
    
    /**
     * Get the total seconds
     * 
     * @return the total seconds
     */    
    virtual int getSecondsTotal() = 0;
    
    /**
     * Get string version
     * 
     * @return the string version
     */
    virtual const char* getVersionString() = 0;    
    
    /**
     * Get speed string
     * 
     * @return the speed string
     */
    virtual const char* getSpeedString() = 0;
        
    /**
     * Get sid model string
     * 
     * @return the sid model string
     */    
    virtual const char* getSidModelString() = 0;
    
    /**
     * Get clock frequnecy string
     * 
     * @return the clock frequency string
     */      
    virtual const char* getClockFreqString() = 0;
    
    // SidTuneMod
    /**
     * Create MD5
     * 
     * @param myMD5 MD5 to create
     */
    virtual void createMD5(MD5&) = 0;
    
    /**
     * Get last CIA1 TA value of C64
     * 
     * @return the last CIA1 TA value
     */    
    virtual int getCia1TA() = 0;
    
    /**
     * Get clock speed     
     *  
     * @return the clock speed (-1=not implemented)
     */
    virtual int getClockSpeed() = 0;    
    
    /**
     * Get the name of Sid Library managed
     * 
     * @return the name of sid library managed
     */
    virtual const char* getName() const = 0;
    
    /**
     * Get the enum of Sid Library managed
     * 
     * @return the enum of sid library managed
     */
    virtual const int getEnum() const = 0;    
   
    /**
     * Get error string message
     * 
     * @return error string message
     */
    const char* getErrorString() const {
       return errorString;
    } 
    
    
    // sid tune wrapper
    
    /**
     * Get the max songs
     * 
     * @return the max songs
     */ 
    virtual int getMaxSongs() const = 0;
        
    /**
     * Get the max sid file length
     * 
     * @return the max sid file length
     */ 
    virtual unsigned long int getMaxSidFileLen() const = 0;   
    
    /**
     * Open the give file
     * 
     * @param fileName the name of file
     * @return true if operation is ok 
     */ 
    virtual bool open(const QString&) = 0;   
    
    /**
     * Load the file in the given buffer
     * 
     * @paraM buf the buffer
     * @param bufLen the length of buffer
     * @return true if the operatuion is ok
     */ 
    virtual bool load(void*, unsigned long int) = 0;
    
    /**
     * Save the file as PSID
     * 
     * @param fileName the fileName to used
     * @param overWrite true if over write
     */ 
    virtual bool savePSID(const QString&, bool) = 0;
      
    /**
     * Update information
     */ 
    virtual void updateInfo() = 0;
    
   /**
    * Get the load address
    * 
    * @return the load address
    */ 
   virtual int getLoadAddr() const = 0;

   /**
    * Get the init address
    * 
    * @return the init address
    */
   virtual int getInitAddr() const = 0;

   /**
    * Get the play address
    * 
    * @return the play address
    */ 
   virtual int getPlayAddr() const = 0;

   /**
    * Get the number of songs
    * 
    * @return the number of songs
    */ 
   virtual int getSongs() const = 0;

   /**
    * Get the current song
    * 
    * @return the current song
    */ 
   virtual int getCurrentSong() const = 0;

   /**
    * Get thwe number of starting song
    *  
    * @return the starting song
    */ 
   virtual int getStartSong() const = 0;

   /**
    * Get the info string of the given index
    * 
    * @param i the index
    * @return the info string
    */ 
   virtual const char* getInfoString(int i) const = 0;

   /**
    * Get numebr of info string
    * 
    * @return the number of imput string
    */ 
   virtual int getInfoStringsNum() const = 0;

   /**
    * Get the status string
    * 
    * @return the status string
    */ 
   virtual const char* getStatusString() const = 0;

   /**
    * Get the speed string
    * 
    * @return the speed string
    */
   virtual const char* getSpeedString2() const = 0;

   /**
    * Get the format string
    * 
    * @return the formal string
    */ 
   virtual const char* getFormatString() const = 0;    
  
   /**
    * Get status
    * 
    * @return the status
    */ 
   virtual bool getStatus() = 0;
   
   /**
    * Get MD5
    * 
    * @return the MD5
    */    
   virtual const QString& getMD5_Digest() = 0;
   
   /**
    * Get the Sid Tune
    * 
    * @return the sid tune
    */ 
   SidLibrary* getSidTune() {
     return this; 
   }  
   
   /**
    * Get the data buffer
    * 
    * @return the data buffer
    */ 
   virtual const unsigned char *getDataBuffer() = 0;

   /**
    * Get the data length
    * 
    * @return the data length
    */
   virtual int getDataLength() = 0;
};

#endif  // SIDLIBRARY_H